@@ -27,14 +27,20 @@
  */
 
 
+#ifdef KSR_PTHREAD_MUTEX_SHARED
+#define HAVE_PTHREAD
+#include <pthread.h>
+#endif
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <strings.h>
+#include <stdint.h>
 
 #include "globals.h"
 #include "dprint.h"
 #include "pvar.h"
+#include "strutils.h"
 
 static void log_callid_set(sip_msg_t *msg);
 
@@ -451,6 +457,36 @@ static pv_elem_t *log_prefix_pvs = NULL;
 static char log_prefix_buf[LOG_PREFIX_SIZE];
 static str log_prefix_str = STR_NULL;
 
+void log_init(void)
+{
+	struct addrinfo hints;
+	struct addrinfo *info = NULL;
+	int gai_result;
+	char hostname[1024];
+
+	hostname[1023] = '\0';
+	gethostname (hostname, 1023);
+
+	memset (&hints, 0, sizeof (hints));
+	hints.ai_family = AF_UNSPEC;    /*either IPV4 or IPV6 */
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_flags = AI_CANONNAME;
+
+	if ((gai_result = getaddrinfo (hostname, 0, &hints, &info)) != 0) {
+		log_fqdn = "?";
+	} else if (info == NULL) {
+		log_fqdn = "?";
+	} else {
+		log_fqdn = strdup (info->ai_canonname);
+	}
+
+	if(info!=NULL) {
+		freeaddrinfo (info);
+	}
+
+	dprint_init_colors();
+}
+
 void log_prefix_init(void)
 {
 	str s;
@@ -497,6 +533,8 @@ static int _ksr_slog_json_flags = 0;
 #define KSR_SLOGJSON_FL_APPPREFIX (1<<2)
 #define KSR_SLOGJSON_FL_NOAPPPREFIXMSG (1<<3)
 #define KSR_SLOGJSON_FL_CALLID (1<<4)
+#define KSR_SLOGJSON_FL_MSGJSON (1<<5)
+#define KSR_SLOGJSON_FL_PRFJSONFLD (1<<6)
 
 
 #define LOGV_CALLID_STR (((_ksr_slog_json_flags & KSR_SLOGJSON_FL_CALLID) \
@@ -504,112 +542,48 @@ static int _ksr_slog_json_flags = 0;
 #define LOGV_CALLID_LEN (((_ksr_slog_json_flags & KSR_SLOGJSON_FL_CALLID) \
 			&& (log_callid_str.len>0))?log_callid_str.len:0)
 
-static void ksr_slog_json_str_escape(str *s_in, str *s_out, int *emode)
-{
-	char *p1, *p2;
-	int len = 0;
-	char token;
-	int i;
-
-	s_out->len = 0;
-	if (!s_in || !s_in->s) {
-		s_out->s = strdup("");
-		*emode = 1;
-		return;
-	}
-	for(i = 0; i < s_in->len; i++) {
-		if (strchr("\"\\\b\f\n\r\t", s_in->s[i])) {
-			len += 2;
-		} else if (s_in->s[i] < 32) {
-			len += 6;
-		} else {
-			len++;
-		}
-	}
-	if(len == s_in->len) {
-		s_out->s = s_in->s;
-		s_out->len = s_in->len;
-		*emode = 0;
-		return;
-	}
-
-	s_out->s = (char*)malloc(len + 2);
-	if (!s_out->s) {
-		return;
-	}
-	*emode = 1;
-
-	p2 = s_out->s;
-	p1 = s_in->s;
-	while (p1 < s_in->s + s_in->len) {
-		if ((unsigned char) *p1 > 31 && *p1 != '\"' && *p1 != '\\') {
-			*p2++ = *p1++;
-		} else {
-			*p2++ = '\\';
-			switch (token = *p1++) {
-			case '\\':
-				*p2++ = '\\';
-				break;
-			case '\"':
-				*p2++ = '\"';
-				break;
-			case '\b':
-				*p2++ = 'b';
-				break;
-			case '\f':
-				*p2++ = 'f';
-				break;
-			case '\n':
-				*p2++ = 'n';
-				break;
-			case '\r':
-				*p2++ = 'r';
-				break;
-			case '\t':
-				*p2++ = 't';
-				break;
-			default:
-				/* escape and print */
-				snprintf(p2, 6, "u%04x", token);
-				p2 += 5;
-				break;
-			}
-		}
-	}
-	*p2++ = 0;
-	s_out->len = len;
-	return;
-}
-
 #define KSR_SLOG_SYSLOG_JSON_FMT "{ \"level\": \"%s\", \"module\": \"%s\", \"file\": \"%s\"," \
-	" \"line\": %d, \"function\": \"%s\", %.*s\"logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"line\": %d, \"function\": \"%s\"%.*s%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_SYSLOG_JSON_CFMT "{ \"level\": \"%s\", \"module\": \"%s\", \"file\": \"%s\"," \
-	" \"line\": %d, \"function\": \"%s\", \"callid\": \"%.*s\", \"logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"line\": %d, \"function\": \"%s\", \"callid\": \"%.*s\"%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_SYSLOG_JSON_PFMT "{ \"" NAME ".level\": \"%s\", \"" NAME ".module\": \"%s\", \"" NAME ".file\": \"%s\"," \
-	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", %.*s\"" NAME ".logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\"%.*s%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_SYSLOG_JSON_CPFMT "{ \"" NAME ".level\": \"%s\", \"" NAME ".module\": \"%s\", \"" NAME ".file\": \"%s\"," \
-	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", \"" NAME ".callid\": \"%.*s\", \"" NAME ".logprefix\": \"%.*s\"," \
-	" \"%smessage\": \"%.*s\" }%s"
+	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", \"" NAME ".callid\": \"%.*s\"%s%s%.*s%s," \
+	" \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_STDERR_JSON_FMT "{ \"idx\": %d, \"pid\": %d, \"level\": \"%s\"," \
 	" \"module\": \"%s\", \"file\": \"%s\"," \
-	" \"line\": %d, \"function\": \"%s\", %.*s\"logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"line\": %d, \"function\": \"%s\"%.*s%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_STDERR_JSON_CFMT "{ \"idx\": %d, \"pid\": %d, \"level\": \"%s\"," \
 	" \"module\": \"%s\", \"file\": \"%s\"," \
-	" \"line\": %d, \"function\": \"%s\", \"callid\": \"%.*s\", \"logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"line\": %d, \"function\": \"%s\", \"callid\": \"%.*s\"%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_STDERR_JSON_PFMT "{ \"" NAME ".idx\": %d, \"" NAME ".pid\": %d, \"" NAME ".level\": \"%s\"," \
 	" \"" NAME ".module\": \"%s\", \"" NAME ".file\": \"%s\"," \
-	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", %.*s\"" NAME ".logprefix\": \"%.*s\", \"%smessage\": \"%.*s\" }%s"
+	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\"%.*s\"%s%s%.*s%s, \"%smessage\": %s%.*s%s }%s"
 
 #define KSR_SLOG_STDERR_JSON_CPFMT "{ \"" NAME ".idx\": %d, \"" NAME ".pid\": %d, \"" NAME ".level\": \"%s\"," \
 	" \"" NAME ".module\": \"%s\", \"" NAME ".file\": \"%s\"," \
-	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", \"" NAME ".callid\": \"%.*s\", \"" NAME ".logprefix\": \"%.*s\"," \
-	" \"%smessage\": \"%.*s\" }%s"
+	" \"" NAME ".line\": %d, \"" NAME ".function\": \"%s\", \"" NAME ".callid\": \"%.*s\"%s%s%.*s%s," \
+	" \"%smessage\": %s%.*s%s }%s"
+
+#ifdef HAVE_PTHREAD
+#define KSR_SLOG_JSON_CEEFMT_TID ",\"tid\":%ju"
+#else
+#define KSR_SLOG_JSON_CEEFMT_TID ""
+#endif
+#define KSR_SLOG_JSON_CEEFMT "{\"time\":\"%s.%09luZ\",\"proc\":{\"id\":\"%d\"" KSR_SLOG_JSON_CEEFMT_TID "},\"pri\":\"%s\",\"subsys\":\"%s\"," \
+        "\"file\":{\"name\":\"%s\",\"line\":%d},\"native\":{\"function\":\"%s\"},\"msg\":%s%.*s%s," \
+        "\"pname\":\"%s\",\"appname\":\"%s\",\"hostname\":\"%s\"}%s"
+
+#define KSR_SLOG_SYSLOG_JSON_CEEFMT "@cee: " KSR_SLOG_JSON_CEEFMT
+
+#define KSR_SLOG_STDERR_JSON_CEEFMT KSR_SLOG_JSON_CEEFMT
 
 void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 {
@@ -623,6 +597,15 @@ void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 	char *prefmsg;
 	const char *efmt;
 	const char *sfmt;
+#define ISO8601_BUF_SIZE 32
+	char iso8601buf[ISO8601_BUF_SIZE + 1];
+	struct timespec _tp;
+	struct tm _tm;
+	char *smb = "\"";
+	char *sme = "\"";
+	char *pmb = "\"";
+	char *pme = "\"";
+	char *prname = ", \"" NAME ".logprefix\": ";
 
 	va_start(arglist, format);
 	n = vsnprintf(obuf + s_in.len, KSR_SLOG_MAX_SIZE - s_in.len, format, arglist);
@@ -640,9 +623,31 @@ void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 		}
 	}
 
-	ksr_slog_json_str_escape(&s_in, &s_out, &emode);
+	if ((!log_cee) && (_ksr_slog_json_flags & KSR_SLOGJSON_FL_MSGJSON)) {
+		if ((s_in.len>1) && (s_in.s[0] == '{') && (s_in.s[s_in.len - 1] == '}')) {
+			s_out = s_in;
+			smb = "";
+			sme = "";
+		} else {
+			smb = "{ \"text\": \"";
+			sme = "\" }";
+		}
+		if((log_prefix_val!=NULL) && (log_prefix_val->len>1)
+				&& (log_prefix_val->s[0] == '{')
+				&& (log_prefix_val->s[log_prefix_val->len - 1] == '}')) {
+			pmb = "";
+			pme = "";
+		} else {
+			pmb = "{ \"text\": \"";
+			pme = "\" }";
+		}
+	}
+
 	if(s_out.s == NULL) {
-		goto error;
+		ksr_str_json_escape(&s_in, &s_out, &emode);
+		if(s_out.s == NULL) {
+			goto error;
+		}
 	}
 
 	if(_ksr_slog_json_flags & KSR_SLOGJSON_FL_APPPREFIX) {
@@ -651,6 +656,7 @@ void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 		} else {
 			prefmsg = NAME ".";
 		}
+		prname = ", \"" NAME ".logprefix\": ";
 		if(_ksr_slog_json_flags & KSR_SLOGJSON_FL_CALLID) {
 			efmt = KSR_SLOG_STDERR_JSON_CPFMT;
 			sfmt = KSR_SLOG_SYSLOG_JSON_CPFMT;
@@ -660,6 +666,7 @@ void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 		}
 	} else {
 		prefmsg = "";
+		prname = ", \"logprefix\": ";
 		if(_ksr_slog_json_flags & KSR_SLOGJSON_FL_CALLID) {
 			efmt = KSR_SLOG_STDERR_JSON_CFMT;
 			sfmt = KSR_SLOG_SYSLOG_JSON_CFMT;
@@ -668,22 +675,59 @@ void ksr_slog_json(ksr_logdata_t *kld, const char *format, ...)
 			sfmt = KSR_SLOG_SYSLOG_JSON_FMT;
 		}
 	}
+	if ((!log_cee) && (_ksr_slog_json_flags & KSR_SLOGJSON_FL_PRFJSONFLD)) {
+		if( (log_prefix_val==NULL) || (log_prefix_val->len<=0)
+				|| ((log_prefix_val->len>1) && (log_prefix_val->s[0] == ','))) {
+			prname = "";
+			pmb = "";
+			pme = "";
+		}
+	}
+	ksr_clock_gettime (&_tp);
+	gmtime_r (&_tp.tv_sec, &_tm);
+	strftime (iso8601buf, ISO8601_BUF_SIZE, "%FT%T", &_tm);
 	if (unlikely(log_stderr)) {
-		if (unlikely(log_color)) dprint_color(kld->v_level);
-		fprintf(stderr,
+		if (unlikely(log_cee)) {
+			fprintf(stderr, KSR_SLOG_STDERR_JSON_CEEFMT,
+			iso8601buf, _tp.tv_nsec, my_pid(),
+#ifdef HAVE_PTHREAD
+                        (uintmax_t)pthread_self(),
+#endif
+                        kld->v_lname,
+			kld->v_mname, kld->v_fname, kld->v_fline, kld->v_func, smb, s_out.len, s_out.s, sme,
+			"kamailio", log_name!=0?log_name:"kamailio", log_fqdn,
+			(_ksr_slog_json_flags & KSR_SLOGJSON_FL_NOLOGNL)?"":"\n");
+		} else {
+			if (unlikely(log_color)) dprint_color(kld->v_level);
+			fprintf(stderr,
 				efmt, process_no, my_pid(),
 				kld->v_lname, kld->v_mname, kld->v_fname, kld->v_fline,
 				kld->v_func, LOGV_CALLID_LEN, LOGV_CALLID_STR,
-				LOGV_PREFIX_LEN, LOGV_PREFIX_STR, prefmsg, s_out.len, s_out.s,
+				prname, pmb, LOGV_PREFIX_LEN, LOGV_PREFIX_STR, pme,
+				prefmsg, smb, s_out.len, s_out.s, sme,
 				(_ksr_slog_json_flags & KSR_SLOGJSON_FL_NOLOGNL)?"":"\n");
-		if (unlikely(log_color)) dprint_color_reset();
+			if (unlikely(log_color)) dprint_color_reset();
+		}
 	} else {
-		_km_log_func(kld->v_facility,
+		if (unlikely(log_cee)) {
+			_km_log_func(kld->v_facility, KSR_SLOG_SYSLOG_JSON_CEEFMT,
+			iso8601buf, _tp.tv_nsec, my_pid(),
+#ifdef HAVE_PTHREAD
+                        pthread_self(),
+#endif
+                        kld->v_lname,
+			kld->v_mname, kld->v_fname, kld->v_fline, kld->v_func, smb, s_out.len, s_out.s, sme,
+			"kamailio", log_name!=0?log_name:"kamailio", log_fqdn,
+			(_ksr_slog_json_flags & KSR_SLOGJSON_FL_NOLOGNL)?"":"\n");
+		} else {
+			_km_log_func(kld->v_facility,
 				sfmt,
 				kld->v_lname, kld->v_mname, kld->v_fname, kld->v_fline,
 				kld->v_func, LOGV_CALLID_LEN, LOGV_CALLID_STR,
-				LOGV_PREFIX_LEN, LOGV_PREFIX_STR, prefmsg, s_out.len, s_out.s,
+				prname, pmb, LOGV_PREFIX_LEN, LOGV_PREFIX_STR, pme,
+				prefmsg, smb, s_out.len, s_out.s, sme,
 				(_ksr_slog_json_flags & KSR_SLOGJSON_FL_NOLOGNL)?"":"\n");
+		}
 	}
 	if(emode && s_out.s) {
 		free(s_out.s);
@@ -716,12 +760,6 @@ void ksr_slog_init(char *ename)
 			_km_log_engine_data = p + 1;
 			while (*p) {
 				switch (*p) {
-					case 'M':
-						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_STRIPMSGNL;
-					break;
-					case 'N':
-						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_NOLOGNL;
-					break;
 					case 'a':
 						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_APPPREFIX;
 					break;
@@ -731,6 +769,21 @@ void ksr_slog_init(char *ename)
 					case 'c':
 						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_CALLID;
 					break;
+					case 'j':
+						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_MSGJSON;
+					break;
+					case 'M':
+						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_STRIPMSGNL;
+					break;
+					case 'N':
+						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_NOLOGNL;
+					break;
+					case 'p':
+						_ksr_slog_json_flags |= KSR_SLOGJSON_FL_PRFJSONFLD;
+					break;
+					case 'U':
+						log_cee = 1;
+					break;
 				}
 				p++;
 			}
