@@ -251,6 +251,50 @@ int fixup_free_regexp_regexp(void** param, int param_no)
 	return fixup_free_regexp_null(param, 1);
 }
 
+int fixup_igp_regexp(void** param, int param_no)
+{
+	struct regex_fixup* re;
+
+	if (param_no == 1) {
+		return fixup_igp_null(param, param_no);
+	}
+	if (param_no == 2) {
+		if ((re=pkg_malloc(sizeof(*re))) ==0) {
+			PKG_MEM_ERROR;
+			goto error;
+		}
+		if (regcomp(&re->regex, *param,
+					REG_EXTENDED|REG_ICASE|REG_NEWLINE))
+			goto error;
+		re->orig = *param;
+		*param = re;
+	}
+	return 0;
+error:
+	if (re)
+		pkg_free(re);
+	return E_UNSPEC;
+}
+
+int fixup_free_igp_regexp(void** param, int param_no)
+{
+	struct regex_fixup* re;
+
+	if (param_no == 1) {
+		return fixup_free_igp_null(param, param_no);
+	}
+	if (param_no == 2) {
+		if (*param) {
+			re = *param;
+			*param = re->orig;
+			regfree(&re->regex);
+			pkg_free(re);
+		}
+	}
+	return 0;
+}
+
+
 /* fixup_pvar_*() has to be written "by hand", since
    it needs to save the original pointer (the fixup users expects
    a pointer to the pv_spec_t in *param and hence the original value
@@ -744,6 +788,7 @@ int fixup_free_vstr_all(void** param, int param_no)
 	pv_elem_free_all((pv_elem_t*)(*param));
 	return 0;
 }
+
 /**
  *
  */
@@ -755,3 +800,103 @@ int fixup_get_vstr_buf(sip_msg_t *msg, pv_elem_t *p, char *buf, int blen)
 	}
 	return -1;
 }
+
+/**
+ *
+ */
+int fixup_ssi(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+			return fixup_spve_null(param, 1);
+		case 3:
+			return fixup_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
+
+/**
+ *
+ */
+int fixup_free_ssi(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+			return fixup_free_spve_null(param, 1);
+		case 3:
+			return fixup_free_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
+
+/**
+ *
+ */
+int fixup_sssi(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+		case 3:
+			return fixup_spve_null(param, 1);
+		case 4:
+			return fixup_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
+
+/**
+ *
+ */
+int fixup_free_sssi(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+		case 3:
+			return fixup_free_spve_null(param, 1);
+		case 4:
+			return fixup_free_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
+
+/**
+ *
+ */
+int fixup_ssii(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+			return fixup_spve_null(param, 1);
+		case 3:
+		case 4:
+			return fixup_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
+
+/**
+ *
+ */
+int fixup_free_ssii(void** param, int param_no)
+{
+	switch(param_no) {
+		case 1:
+		case 2:
+			return fixup_free_spve_null(param, 1);
+		case 3:
+		case 4:
+			return fixup_free_igp_null(param, 1);
+		default:
+			return E_UNSPEC;
+	}
+}
