@@ -2667,6 +2667,7 @@ endofpacket:
 	/*DBG("parse_via: rest=<%s>\n", tmp);*/
 
 	vb->error = PARSE_OK;
+	vb->bstart = buffer;
 	vb->bsize = tmp - buffer;
 	if(vb->port_str.s) {
 		vb->port = str2s(vb->port_str.s, vb->port_str.len, &err);
@@ -2680,6 +2681,7 @@ endofpacket:
 nextvia:
 	DBG("parsing via: next via\n");
 	vb->error = PARSE_OK;
+	vb->bstart = buffer;
 	vb->bsize = tmp - buffer;
 	if(vb->port_str.s) {
 		vb->port = str2s(vb->port_str.s, vb->port_str.len, &err);
@@ -2701,11 +2703,11 @@ nextvia:
 
 error:
 	if(end > buffer) {
-		LM_ERR("parsing via on: <%.*s>\n", (int)(end - buffer), ZSW(buffer));
+		LM_ERR("parsing via on: <%.*s>\n", (int)(end - buffer), ZSW(ksr_buf_oneline(buffer, (int)(end - buffer))));
 	}
 	if((tmp > buffer) && (tmp < end)) {
 		LM_ERR("parse error, parsed so far:<%.*s>\n", (int)(tmp - buffer),
-				ZSW(buffer));
+				ZSW(ksr_buf_oneline(buffer, (int)(tmp - buffer))));
 	} else {
 		LM_ERR("via parse error\n");
 	}
