@@ -841,7 +841,7 @@ again:
 #endif
 		p+=skip;
 		rec_name_len=strlen(rec_name);
-		if (unlikely(rec_name_len>255)){
+		if (unlikely(rec_name_len>MAX_DNS_NAME-2)){
 			LM_ERR("dn_expand(rec_name): name too long (%d)\n",
 					rec_name_len);
 			goto error;
@@ -1388,9 +1388,9 @@ end:
  *         tried      - bitmap used to keep track of the already tried records
  *                      (no more then sizeof(tried)*8 valid records are
  *                      ever walked
- *         srv_name   - if succesfull, it will be set to the selected record
+ *         srv_name   - if successful, it will be set to the selected record
  *                      srv name (naptr repl.)
- *         proto      - if succesfull it will be set to the selected record
+ *         proto      - if successful it will be set to the selected record
  *                      protocol
  * returns  0 if no more records found or a pointer to the selected record
  *  and sets  protocol and srv_name
