@@ -1502,7 +1502,8 @@ static int build_iface_list(void)
 			}
 
 			if(strlen(ifaces[index].name)==0 && strlen(name)>0) {
-				strncpy(ifaces[index].name, name, MAX_IF_LEN-1);
+				memcpy(ifaces[index].name, name, MAX_IF_LEN-1);
+				ifaces[index].name[MAX_IF_LEN-1] = '\0';
 			}
 
 			ifaces[index].index = index;
@@ -1713,7 +1714,7 @@ static int fix_hostname(str* name, struct ip_addr* address, str* address_str,
 	}
 	
 #ifdef USE_MCAST
-	/* Check if it is an multicast address and
+	/* Check if it is a multicast address and
 	 * set the flag if so
 	 */
 	if (is_mcast(address)){
