@@ -104,6 +104,9 @@ struct socket_info* grep_sock_info_by_port(unsigned short port,
 struct socket_info* find_si(struct ip_addr* ip, unsigned short port,
 												unsigned short proto);
 socket_info_t* ksr_get_socket_by_name(str *sockname);
+socket_info_t* ksr_get_socket_by_listen(str *sockstr);
+socket_info_t* ksr_get_socket_by_advertise(str *sockstr);
+socket_info_t* ksr_get_socket_by_index(int idx);
 
 struct socket_info** get_sock_info_list(unsigned short proto);
 
@@ -151,8 +154,10 @@ inline static struct socket_info* get_first_socket(void)
 
 /* structure to break down 'proto:host:port' */
 typedef struct _sr_phostp {
+	str sproto;
 	int proto;
 	str host;
+	str sport;
 	int port;
 } sr_phostp_t;
 
@@ -161,4 +166,7 @@ int parse_protohostport(str* ins, sr_phostp_t *r);
 
 unsigned int ipv6_get_netif_scope(char *ipval);
 
+int ksr_sockets_no_get(void);
+void ksr_sockets_index(void);
+
 #endif
