@@ -270,10 +270,10 @@
 #define USE_NAPTR_STR ""
 #endif
 
-#ifdef USE_DST_BLACKLIST
-#define USE_DST_BLACKLIST_STR ", USE_DST_BLACKLIST"
+#ifdef USE_DST_BLOCKLIST
+#define USE_DST_BLOCKLIST_STR ", USE_DST_BLOCKLIST"
 #else
-#define USE_DST_BLACKLIST_STR ""
+#define USE_DST_BLOCKLIST_STR ""
 #endif
 
 #ifdef NO_SIG_DEBUG
@@ -312,10 +312,10 @@
 #define USE_DNS_CACHE_STATS_STR ""
 #endif
 
-#ifdef USE_DST_BLACKLIST_STATS
-#define USE_DST_BLACKLIST_STATS_STR ", USE_DST_BLACKLIST_STATS"
+#ifdef USE_DST_BLOCKLIST_STATS
+#define USE_DST_BLOCKLIST_STATS_STR ", USE_DST_BLOCKLIST_STATS"
 #else
-#define USE_DST_BLACKLIST_STATS_STR ""
+#define USE_DST_BLOCKLIST_STATS_STR ""
 #endif
 
 #ifdef KSR_PTHREAD_MUTEX_SHARED
@@ -336,9 +336,9 @@
 	TIMER_DEBUG_STR USE_FUTEX_STR \
 	FAST_LOCK_STR NOSMP_STR USE_PTHREAD_MUTEX_STR USE_POSIX_SEM_STR \
 	USE_SYSV_SEM_STR USE_COMP_STR USE_DNS_CACHE_STR USE_DNS_FAILOVER_STR \
-	DNS_WATCHDOG_SUPPORT_STR USE_NAPTR_STR USE_DST_BLACKLIST_STR \
+	DNS_WATCHDOG_SUPPORT_STR USE_NAPTR_STR USE_DST_BLOCKLIST_STR \
 	HAVE_RESOLV_RES_STR SYSLOG_CALLBACK_SUPPORT_STR MYSQL_FAKE_NULL_STR \
-	USE_DST_BLACKLIST_STATS_STR USE_DNS_CACHE_STATS_STR \
+	USE_DST_BLOCKLIST_STATS_STR USE_DNS_CACHE_STATS_STR \
 	KSR_PTHREAD_MUTEX_SHARED_STR
 
 
