@@ -56,6 +56,7 @@ static int w_app_jsdt_run3(sip_msg_t *msg, char *func, char *p1, char *p2,
 static int fixup_jsdt_run(void** param, int param_no);
 
 extern str _sr_jsdt_load_file;
+extern int _sr_jsdt_mode;
 
 /* clang-format off */
 static cmd_export_t cmds[]={
@@ -79,6 +80,7 @@ static cmd_export_t cmds[]={
 
 static param_export_t params[]={
 	{"load", PARAM_STR, &_sr_jsdt_load_file},
+	{"mode", PARAM_INT, &_sr_jsdt_mode},
 	{0, 0, 0}
 };
 
@@ -117,7 +119,10 @@ static int mod_init(void)
  */
 static int child_init(int rank)
 {
-	return jsdt_sr_init_child();
+	if(rank==PROC_INIT) {
+		return 0;
+	}
+	return jsdt_sr_init_child(rank);
 }
 
 /**
