@@ -42,6 +42,7 @@ MODULE_VERSION
 
 static int async_workers = 1;
 static int async_ms_timer = 0;
+static int async_return = 0;
 
 static int mod_init(void);
 static int child_init(int);
@@ -56,6 +57,9 @@ static int w_async_ms_route(sip_msg_t *msg, char *rt, char *sec);
 static int fixup_async_route(void **param, int param_no);
 
 static int w_async_task_route(sip_msg_t *msg, char *rt, char *p2);
+static int w_async_task_group_route(sip_msg_t *msg, char *rt, char *gr);
+static int w_async_task_data(sip_msg_t *msg, char *rt, char *pdata);
+static int w_async_task_group_data(sip_msg_t *msg, char *rt, char *gr, char *pdata);
 static int fixup_async_task_route(void **param, int param_no);
 
 /* tm */
@@ -73,22 +77,36 @@ static cmd_export_t cmds[]={
 		0, REQUEST_ROUTE|FAILURE_ROUTE},
 	{"async_task_route", (cmd_function)w_async_task_route, 1, fixup_async_task_route,
 		0, REQUEST_ROUTE|FAILURE_ROUTE},
+	{"async_task_group_route", (cmd_function)w_async_task_group_route, 2, fixup_async_task_route,
+		0, REQUEST_ROUTE|FAILURE_ROUTE},
+	{"async_task_data", (cmd_function)w_async_task_data, 2, fixup_async_task_route,
+		0, ANY_ROUTE},
+	{"async_task_group_data", (cmd_function)w_async_task_group_data, 3, fixup_async_task_route,
+		0, ANY_ROUTE},
+
 	{0, 0, 0, 0, 0, 0}
 };
 
 static param_export_t params[]={
 	{"workers",     INT_PARAM,   &async_workers},
 	{"ms_timer",    INT_PARAM,   &async_ms_timer},
+	{"return",      INT_PARAM,   &async_return},
 	{0, 0, 0}
 };
 
+static pv_export_t mod_pvs[] = {
+	{ {"async", sizeof("async")-1}, PVT_OTHER, pv_get_async, 0,
+		pv_parse_async_name, 0, 0, 0 },
+	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
+};
+
 struct module_exports exports = {
 	"async",
 	DEFAULT_DLFLAGS, /* dlopen flags */
 	cmds,
 	params,
 	0,              /* exported RPC methods */
-	0,              /* exported pseudo-variables */
+	mod_pvs,        /* exported pseudo-variables */
 	0,              /* response function */
 	mod_init,       /* module initialization function */
 	child_init,     /* per child init function */
@@ -102,6 +120,12 @@ struct module_exports exports = {
  */
 static int mod_init(void)
 {
+	/* init faked sip msg */
+	if(faked_msg_init()<0) {
+		LM_ERR("failed to iit local sip msg\n");
+		return -1;
+	}
+
 	if(load_tm_api(&tmb) == -1) {
 		LM_ERR("cannot load the TM-functions. Missing TM module?\n");
 		return -1;
@@ -209,7 +233,7 @@ static int w_async_sleep(sip_msg_t *msg, char *sec, char *str2)
 		if(async_sleep(msg, s, ap->u.paction->next, NULL) < 0)
 			return -1;
 		/* force exit in config */
-		return 0;
+		return async_return;
 	}
 
 	return -1;
@@ -249,7 +273,7 @@ static int w_async_ms_sleep(sip_msg_t *msg, char *sec, char *str2)
 		if(async_ms_sleep(msg, s, ap->u.paction->next, NULL) < 0)
 			return -1;
 		/* force exit in config */
-		return 0;
+		return async_return;
 	}
 
 	return -1;
@@ -312,7 +336,7 @@ int ki_async_route(sip_msg_t *msg, str *rn, int s)
 	if(async_sleep(msg, s, act, rn) < 0)
 		return -1;
 	/* force exit in config */
-	return 0;
+	return async_return;
 }
 
 /**
@@ -348,7 +372,7 @@ int ki_async_ms_route(sip_msg_t *msg, str *rn, int s)
 	if(async_ms_sleep(msg, s, act, rn) < 0)
 		return -1;
 	/* force exit in config */
-	return 0;
+	return async_return;
 }
 
 /**
@@ -426,7 +450,7 @@ static int fixup_async_route(void **param, int param_no)
 /**
  *
  */
-int ki_async_task_route(sip_msg_t *msg, str *rn)
+int ki_async_task_group_route(sip_msg_t *msg, str *rn, str *gn)
 {
 	cfg_action_t *act = NULL;
 	int ri;
@@ -453,16 +477,24 @@ int ki_async_task_route(sip_msg_t *msg, str *rn)
 		}
 	}
 
-	if(async_send_task(msg, act, rn) < 0)
+	if(async_send_task(msg, act, rn, gn) < 0)
 		return -1;
 	/* force exit in config */
-	return 0;
+	return async_return;
 }
 
 /**
  *
  */
-static int w_async_task_route(sip_msg_t *msg, char *rt, char *sec)
+int ki_async_task_route(sip_msg_t *msg, str *rn)
+{
+	return  ki_async_task_group_route(msg, rn, NULL);
+}
+
+/**
+ *
+ */
+static int w_async_task_route(sip_msg_t *msg, char *rt, char *p2)
 {
 	str rn;
 
@@ -476,6 +508,29 @@ static int w_async_task_route(sip_msg_t *msg, char *rt, char *sec)
 	return ki_async_task_route(msg, &rn);
 }
 
+/**
+ *
+ */
+static int w_async_task_group_route(sip_msg_t *msg, char *rt, char *gr)
+{
+	str rn;
+	str gn;
+
+	if(msg == NULL)
+		return -1;
+
+	if(fixup_get_svalue(msg, (gparam_t *)rt, &rn) != 0) {
+		LM_ERR("no async route block name\n");
+		return -1;
+	}
+	if(fixup_get_svalue(msg, (gparam_t *)gr, &gn) != 0) {
+		LM_ERR("no async group name\n");
+		return -1;
+	}
+
+	return ki_async_task_group_route(msg, &rn, &gn);
+}
+
 /**
  *
  */
@@ -487,7 +542,7 @@ static int fixup_async_task_route(void **param, int param_no)
 		return -1;
 	}
 
-	if(param_no == 1) {
+	if(param_no == 1 || param_no == 2 || param_no == 2) {
 		if(fixup_spve_null(param, 1) < 0)
 			return -1;
 		return 0;
@@ -495,6 +550,97 @@ static int fixup_async_task_route(void **param, int param_no)
 	return 0;
 }
 
+/**
+ *
+ */
+int ki_async_task_group_data(sip_msg_t *msg, str *rn, str *gn, str *sdata)
+{
+	cfg_action_t *act = NULL;
+	int ri;
+	sr_kemi_eng_t *keng = NULL;
+
+	keng = sr_kemi_eng_get();
+	if(keng == NULL) {
+		ri = route_lookup(&main_rt, rn->s);
+		if(ri >= 0) {
+			act = main_rt.rlist[ri];
+			if(act == NULL) {
+				LM_ERR("empty action lists in route block [%.*s]\n", rn->len,
+						rn->s);
+				return -1;
+			}
+		} else {
+			LM_ERR("route block not found: %.*s\n", rn->len, rn->s);
+			return -1;
+		}
+	}
+
+	if(async_send_data(msg, act, rn, gn, sdata) < 0)
+		return -1;
+	/* ok */
+	return 1;
+}
+
+/**
+ *
+ */
+int ki_async_task_data(sip_msg_t *msg, str *rn, str *sdata)
+{
+	return  ki_async_task_group_data(msg, rn, NULL, sdata);
+}
+
+
+/**
+ *
+ */
+static int w_async_task_data(sip_msg_t *msg, char *rt, char *pdata)
+{
+	str rn;
+	str sdata;
+
+	if(msg == NULL)
+		return -1;
+
+	if(fixup_get_svalue(msg, (gparam_t *)rt, &rn) != 0) {
+		LM_ERR("no async route block name\n");
+		return -1;
+	}
+	if(fixup_get_svalue(msg, (gparam_t *)pdata, &sdata) != 0) {
+		LM_ERR("no async data\n");
+		return -1;
+	}
+
+	return ki_async_task_data(msg, &rn, &sdata);
+}
+
+/**
+ *
+ */
+static int w_async_task_group_data(sip_msg_t *msg, char *rt, char *gr, char *pdata)
+{
+	str rn;
+	str gn;
+	str sdata;
+
+	if(msg == NULL)
+		return -1;
+
+	if(fixup_get_svalue(msg, (gparam_t *)rt, &rn) != 0) {
+		LM_ERR("no async route block name\n");
+		return -1;
+	}
+	if(fixup_get_svalue(msg, (gparam_t *)gr, &gn) != 0) {
+		LM_ERR("no async group name\n");
+		return -1;
+	}
+	if(fixup_get_svalue(msg, (gparam_t *)pdata, &sdata) != 0) {
+		LM_ERR("no async data\n");
+		return -1;
+	}
+
+	return ki_async_task_group_data(msg, &rn, &gn, &sdata);
+}
+
 /**
  *
  */
@@ -515,6 +661,31 @@ static sr_kemi_t sr_kemi_async_exports[] = {
 		{ SR_KEMIP_STR, SR_KEMIP_NONE, SR_KEMIP_NONE,
 			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
 	},
+	{ str_init("async"), str_init("task_group_route"),
+		SR_KEMIP_INT, ki_async_task_group_route,
+		{ SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_NONE,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
+	{ str_init("async"), str_init("task_data"),
+		SR_KEMIP_INT, ki_async_task_data,
+		{ SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_NONE,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
+	{ str_init("async"), str_init("task_group_data"),
+		SR_KEMIP_INT, ki_async_task_group_data,
+		{ SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_STR,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
+	{ str_init("async"), str_init("get_gname"),
+		SR_KEMIP_XVAL, ki_async_get_gname,
+		{ SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
+	{ str_init("async"), str_init("get_data"),
+		SR_KEMIP_XVAL, ki_async_get_data,
+		{ SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE,
+			SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
+	},
 
 	{ {0, 0}, {0, 0}, 0, NULL, { 0, 0, 0, 0, 0, 0 } }
 };
