@@ -40,6 +40,7 @@
 #include "../../core/str.h"
 #include "../../core/pvar.h"
 #include "../../core/ut.h"
+#include "../../core/trim.h"
 #include "../../core/script_cb.h"
 #include "../../core/parser/digest/digest.h"
 #include "../../core/parser/parse_from.h"
@@ -121,7 +122,7 @@ int parse_param_stop(unsigned int type, void *val);
 
 /* Local global variables */
 static CallControlSocket callcontrol_socket = {
-    "/var/run/callcontrol/socket", // name
+    "/run/callcontrol/socket", // name
     -1,                            // sock
     500,                           // timeout in 500 miliseconds if there is no answer
     0,                             // time of the last failure
@@ -351,41 +352,6 @@ parse_param_stop(unsigned int type, void *val) {
 }
 
 
-// Functions dealing with strings
-//
-
-// returns string with whitespace trimmed from left end
-static inline void
-ltrim(str *string)
-{
-    while (string->len>0 && isspace((int)*(string->s))) {
-        string->len--;
-        string->s++;
-    }
-}
-
-// returns string with whitespace trimmed from right end
-static inline void
-rtrim(str *string)
-{
-    char *ptr;
-
-    ptr = string->s + string->len - 1;
-    while (string->len>0 && (*ptr==0 || isspace((int)*ptr))) {
-        string->len--;
-        ptr--;
-    }
-}
-
-// returns string with whitespace trimmed from both ends
-static inline void
-trim(str *string)
-{
-    ltrim(string);
-    rtrim(string);
-}
-
-
 // Message checking and parsing
 //
 
