@@ -19,10 +19,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <time.h>
-#include <string.h>
+
 #include <stdlib.h>
-#include <sys/time.h>
 
 #include "cnxcc.h"
 
