@@ -112,59 +112,59 @@ static cmd_export_t cmds[]={
 	{"append_branch", (cmd_function)w_append_branch, 0, 0,
 		0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"append_branch", (cmd_function)w_append_branch, 1, fixup_spve_null,
-		0, REQUEST_ROUTE | FAILURE_ROUTE },
+		fixup_free_spve_null, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"append_branch", (cmd_function)w_append_branch, 2, fixup_spve_spve,
-		0, REQUEST_ROUTE | FAILURE_ROUTE },
+		fixup_free_spve_spve, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"send_udp", (cmd_function)w_send_udp, 0, 0,
 		0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"send_udp", (cmd_function)w_send_udp, 1, fixup_spve_null,
-		0, REQUEST_ROUTE | FAILURE_ROUTE },
+		fixup_free_spve_null, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"send_tcp", (cmd_function)w_send_tcp, 0, 0,
 		0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"send_tcp", (cmd_function)w_send_tcp, 1, fixup_spve_null,
-		0, REQUEST_ROUTE | FAILURE_ROUTE },
+		fixup_free_spve_null, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"send_data", (cmd_function)w_send_data, 2, fixup_spve_spve,
-		0, ANY_ROUTE },
+		fixup_free_spve_spve, ANY_ROUTE },
 	{"sendx", (cmd_function)w_sendx, 3, fixup_spve_all,
-		0, ANY_ROUTE },
+		fixup_free_spve_all, ANY_ROUTE },
 	{"is_incoming",    (cmd_function)nio_check_incoming, 0, 0,
 		0, ANY_ROUTE },
 	{"msg_iflag_set", (cmd_function)w_msg_iflag_set,       1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"msg_iflag_reset", (cmd_function)w_msg_iflag_reset,   1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"msg_iflag_is_set", (cmd_function)w_msg_iflag_is_set, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"file_read", (cmd_function)w_file_read,   2, fixup_file_op,
 		0, ANY_ROUTE },
 	{"file_write", (cmd_function)w_file_write, 2, fixup_spve_spve,
-		0, ANY_ROUTE },
-	{"setxflag", (cmd_function)w_setxflag,          1,fixup_igp_null,
-		0, ANY_ROUTE },
-	{"resetxflag", (cmd_function)w_resetxflag,      1,fixup_igp_null,
-		0, ANY_ROUTE },
-	{"isxflagset", (cmd_function)w_isxflagset,      1,fixup_igp_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_spve, ANY_ROUTE },
+	{"setxflag", (cmd_function)w_setxflag,          1, fixup_igp_null,
+		fixup_free_igp_null, ANY_ROUTE },
+	{"resetxflag", (cmd_function)w_resetxflag,      1, fixup_igp_null,
+		fixup_free_igp_null, ANY_ROUTE },
+	{"isxflagset", (cmd_function)w_isxflagset,      1, fixup_igp_null,
+		fixup_free_igp_null, ANY_ROUTE },
 	{"set_send_socket", (cmd_function)w_set_send_socket, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"set_send_socket_name", (cmd_function)w_set_send_socket_name, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"set_recv_socket", (cmd_function)w_set_recv_socket, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"set_recv_socket_name", (cmd_function)w_set_recv_socket_name, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"set_source_address", (cmd_function)w_set_source_address, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 	{"via_add_srvid", (cmd_function)w_via_add_srvid, 1, fixup_igp_null,
-		0, ANY_ROUTE },
+		fixup_free_igp_null, ANY_ROUTE },
 	{"via_add_xavp_params", (cmd_function)w_via_add_xavp_params, 1, fixup_igp_null,
-		0, ANY_ROUTE },
+		fixup_free_igp_null, ANY_ROUTE },
 	{"via_use_xavp_fields", (cmd_function)w_via_use_xavp_fields, 1, fixup_igp_null,
-		0, ANY_ROUTE },
+		fixup_free_igp_null, ANY_ROUTE },
 	{"is_faked_msg", (cmd_function)w_is_faked_msg, 0, 0,
 		0, ANY_ROUTE },
 	{"is_socket_name", (cmd_function)w_is_socket_name, 1, fixup_spve_null,
-		0, ANY_ROUTE },
+		fixup_free_spve_null, ANY_ROUTE },
 
 	{0, 0, 0, 0, 0, 0}
 };
