@@ -32,13 +32,13 @@
 #include "../../core/tcp_options.h"
 #include "../../core/msg_translator.h"
 
-int nio_route_no;
-int nio_min_msg_len;
-int nio_is_incoming;
+extern int nio_route_no;
+extern int nio_min_msg_len;
+extern int nio_is_incoming;
 
-str nio_msg_avp_param;
-int_str nio_msg_avp_name;
-unsigned short nio_msg_avp_type;
+extern str nio_msg_avp_param;
+extern int_str nio_msg_avp_name;
+extern unsigned short nio_msg_avp_type;
 
 int nio_msg_received(sr_event_param_t *evp);
 int nio_msg_sent(sr_event_param_t *evp);
