@@ -36,7 +36,7 @@
 
 /************************ imported from "utils.h"  ***************************/
 
-static inline int strz2int(char *_bp)
+static inline int cpl_strz2int(char *_bp)
 {
 	int _v;
 	char *_p;
@@ -53,33 +53,6 @@ static inline int strz2int(char *_bp)
 }
 
 
-static inline char* trim(char* _s)
-{
-	int len;
-	char* end;
-
-	     /* Null pointer, there is nothing to do */
-	if (!_s) return _s;
-
-	     /* Remove spaces and tabs from the beginning of string */
-	while ((*_s == ' ') || (*_s == '\t')) _s++;
-
-	len = strlen(_s);
-
-        end = _s + len - 1;
-
-	     /* Remove trailing spaces and tabs */
-	while ((*end == ' ') || (*end == '\t')) end--;
-	if (end != (_s + len - 1)) {
-		*(end+1) = '\0';
-	}
-
-	return _s;
-}
-
-
-
-
 /************************ imported from "ac_tm.c"  ***************************/
 
 /* #define USE_YWEEK_U		// Sunday system
@@ -444,7 +417,7 @@ int cpl_tr_parse_interval(cpl_tmrec_p _trp, char *_in)
 {
 	if(!_trp || !_in)
 		return -1;
-	_trp->interval = strz2int(_in);
+	_trp->interval = cpl_strz2int(_in);
 	return 0;
 }
 
