@@ -23,14 +23,19 @@
 #ifndef _REDIS_CONNECTION_H_
 #define _REDIS_CONNECTION_H_
 
+#ifdef WITH_HIREDIS_CLUSTER
+#include <hircluster.h>
+#else
 #ifdef WITH_HIREDIS_PATH
 #include <hiredis/hiredis.h>
 #else
 #include <hiredis.h>
 #endif
+#endif
 
 #include "db_redis_mod.h"
 
+#ifndef WITH_REDIS_CLUSTER
 #define db_redis_check_reply(con, reply, err) do { \
     if (!(reply) && !(con)->con) { \
         LM_ERR("Failed to fetch type entry: no connection to server\n"); \
@@ -49,6 +54,26 @@
         goto err; \
     } \
 } while(0);
+#else
+#define db_redis_check_reply(con, reply, err) do { \
+    if (!(reply) && !(con)->con) { \
+        LM_ERR("Failed to fetch type entry: no connection to server\n"); \
+        goto err; \
+    } \
+    if (!(reply)) { \
+        LM_ERR("Failed to fetch type entry: %s\n", \
+                (con)->con->errstr); \
+        redisClusterFree((con)->con); \
+        (con)->con = NULL; \
+        goto err; \
+    } \
+    if ((reply)->type == REDIS_REPLY_ERROR) { \
+        LM_ERR("Failed to fetch type entry: %s\n", \
+                (reply)->str); \
+        goto err; \
+    } \
+} while(0);
+#endif
 
 typedef struct redis_key redis_key_t;
 
@@ -61,8 +86,11 @@ typedef struct km_redis_con {
     struct db_id* id;
     unsigned int ref;
     struct pool_con* next;
-
+#ifdef WITH_HIREDIS_CLUSTER
+    redisClusterContext *con;
+#else
     redisContext *con;
+#endif
     redis_command_t *command_queue;
     unsigned int append_counter;
     struct str_hash_table tables;
@@ -86,4 +114,8 @@ void db_redis_consume_replies(km_redis_con_t *con);
 void db_redis_free_reply(redisReply **reply);
 const char *db_redis_get_error(km_redis_con_t *con);
 
+#ifdef WITH_HIREDIS_CLUSTER
+void *db_redis_command_argv_to_node(km_redis_con_t *con, redis_key_t *query, cluster_node *node);
+#endif
+
 #endif /* _REDIS_CONNECTION_H_ */
