@@ -71,6 +71,7 @@ extern int       dlg_send_bye;
 extern int       dlg_event_rt[DLG_EVENTRT_MAX];
 extern int       dlg_wait_ack;
 extern int       dlg_enable_dmq;
+extern int       dlg_filter_mode;
 int              spiral_detected = -1;
 
 extern struct rr_binds d_rrb;		/*!< binding to record-routing module */
@@ -1581,6 +1582,18 @@ void dlg_ontimeout(struct dlg_tl *tl)
 	dlg = ((struct dlg_cell*)((char *)(tl) -
 			(unsigned long)(&((struct dlg_cell*)0)->tl)));
 
+	if (dlg_filter_mode & DLG_FILTER_LOCALONLY) {
+		if (dlg->bind_addr[0] == NULL) {
+			LM_DBG("skipping dialog without bind address\n");
+			return;
+		}
+
+		if (lookup_local_socket(&(dlg->bind_addr[0]->sock_str)) == NULL) {
+			LM_DBG("skipping non local dialog\n");
+			return;
+		}
+	}
+
 	/* mark dialog as expired */
 	dlg->dflags |= DLG_FLAG_EXPIRED;
 
