@@ -40,7 +40,7 @@
 /* states of a dialog */
 #define DLG_STATE_UNCONFIRMED  1 /*!< unconfirmed dialog */
 #define DLG_STATE_EARLY        2 /*!< early dialog */
-#define DLG_STATE_CONFIRMED_NA 3 /*!< confirmed dialog without a ACK yet */
+#define DLG_STATE_CONFIRMED_NA 3 /*!< confirmed dialog without an ACK yet */
 #define DLG_STATE_CONFIRMED    4 /*!< confirmed dialog */
 #define DLG_STATE_DELETED      5 /*!< deleted dialog */
 
@@ -92,6 +92,9 @@
 #define DLG_EVENTRT_FAILED   2
 #define DLG_EVENTRT_MAX      3
 
+/* flags for filtering dialogs processing */
+#define DLG_FILTER_LOCALONLY (1<<0) /*!< processing only local dialogs */
+
 /*! internal unique ide per dialog */
 typedef struct dlg_iuid {
 	unsigned int         h_id;		/*!< id in the hash table entry (seq nr in slot) */
@@ -462,7 +465,7 @@ void dlg_ref_helper(dlg_cell_t *dlg, unsigned int cnt, const char *fname,
 void dlg_release(dlg_cell_t *dlg);
 
 /*!
- * \brief Update a dialog state according a event and the old state
+ * \brief Update a dialog state according an event and the old state
  *
  * This functions implement the main state machine that update a dialog
  * state according a processed event and the current state. If necessary
