@@ -66,8 +66,8 @@ void timer_cb(int fd, short kind, void *userp)
 
 	LM_DBG("timeout on socket %d\n", fd);
 
-	rc = curl_multi_socket_action(g->multi,
-                                  CURL_SOCKET_TIMEOUT, 0, &g->still_running);
+	rc = curl_multi_socket_action(g->multi, CURL_SOCKET_TIMEOUT, 0, &g->still_running);
+
 	if (check_mcode(rc, error) < 0) {
 		LM_ERR("curl_multi_socket_action error: %s", error);
 	}
@@ -122,8 +122,7 @@ void event_cb(int fd, short kind, void *userp)
 		LM_DBG("removing handle %p\n", easy);
 		curl_multi_remove_handle(g->multi, easy);
 		curl_easy_cleanup(easy);
-		rc = curl_multi_socket_action(g->multi,
-                                  CURL_SOCKET_TIMEOUT, 0, &g->still_running);
+		rc = curl_multi_socket_action(g->multi, CURL_SOCKET_TIMEOUT, 0, &g->still_running);
 
 	} else {
 		LM_DBG("performing action %d on socket %d\n", action, fd);
@@ -403,6 +402,7 @@ int init_http_multi(struct event_base *evbase, struct http_m_global *wg)
 	curl_multi_setopt(g->multi, CURLMOPT_SOCKETDATA, g);
 	curl_multi_setopt(g->multi, CURLMOPT_TIMERFUNCTION, multi_timer_cb);
 	curl_multi_setopt(g->multi, CURLMOPT_TIMERDATA, g);
+	curl_multi_setopt(g->multi, CURLMOPT_PIPELINING, CURLPIPE_NOTHING);
 
 	return init_http_m_table(hash_size);
 }
@@ -462,6 +462,9 @@ int new_request(str *query, http_m_params_t *query_params, http_multi_cbe_t cb,
 		curl_easy_setopt(cell->easy, CURLOPT_VERBOSE, 1L);
 		curl_easy_setopt(cell->easy, CURLOPT_DEBUGFUNCTION, debug_cb);
 	}
+	if (cell->params.follow_redirect) {
+		curl_easy_setopt(cell->easy, CURLOPT_FOLLOWLOCATION, 1L);
+	}
 	curl_easy_setopt(cell->easy, CURLOPT_ERRORBUFFER, cell->error);
 	curl_easy_setopt(cell->easy, CURLOPT_PRIVATE, cell);
 	curl_easy_setopt(cell->easy, CURLOPT_SSL_VERIFYPEER, cell->params.tls_verify_peer);
@@ -568,6 +571,7 @@ void check_multi_info(struct http_m_global *g)
 	CURLcode res;
 
 	struct http_m_cell *cell;
+	double tmp_time;
 
 	LM_DBG("REMAINING: %d\n", g->still_running);
 	while ((msg = curl_multi_info_read(g->multi, &msgs_left))) {
@@ -576,6 +580,8 @@ void check_multi_info(struct http_m_global *g)
 			res = msg->data.result;
 			curl_easy_getinfo(easy, CURLINFO_PRIVATE, &cell);
 			curl_easy_getinfo(easy, CURLINFO_EFFECTIVE_URL, &eff_url);
+
+
 			LM_DBG("DONE: %s => (%d) %s\n", eff_url, res, cell->error);
 
 			cell = http_m_cell_lookup(easy);
@@ -584,6 +590,22 @@ void check_multi_info(struct http_m_global *g)
 				update_stat(errors, 1);
 				reply_error(cell);
 			} else {
+				
+				if (curl_easy_getinfo(cell->easy, CURLINFO_TOTAL_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.total=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_NAMELOOKUP_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.lookup=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_CONNECT_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.connect=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_REDIRECT_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.redirect=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_APPCONNECT_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.appconnect=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_PRETRANSFER_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.pretransfer=(uint32_t)(tmp_time*1000000);
+				if (curl_easy_getinfo(cell->easy, CURLINFO_STARTTRANSFER_TIME, &tmp_time) == CURLE_OK)
+					cell->reply->time.starttransfer=(uint32_t)(tmp_time*1000000);
+				
 				cell->reply->error[0] = '\0';
 				cell->cb(cell->reply, cell->param);
 
