@@ -48,12 +48,17 @@
 #include "hm_hash.h"
 
 
+#ifndef CURLPIPE_NOTHING
+#define CURLPIPE_NOTHING   0L
+#endif
+
 extern stat_var *requests;
 extern stat_var *replies;
 extern stat_var *errors;
 extern stat_var *timeouts;
 extern int tls_version;
 extern int curl_verbose;
+extern int curl_follow_redirect;
 
 void set_curl_mem_callbacks(void);
 int init_http_multi();
