@@ -21,7 +21,7 @@ struct interim_ccr {
 void credit_control_session_callback(int event, void* session);
 void remove_aaa_session(str *session_id);
 int Ro_Send_CCR(struct sip_msg *msg, struct dlg_cell *dlg, int dir, int reservation_units, 
-	    str *incoming_trunk_id, str *outgoing_trunk_id, str *enb_cell_id, cfg_action_t* action, unsigned int tindex, unsigned int tlabel);
+	    str *incoming_trunk_id, str *outgoing_trunk_id, str *enb_cell_id, void* action, unsigned int tindex, unsigned int tlabel);
 long get_current_time_micro();
 void send_ccr_interim(struct ro_session* ro_session, unsigned int used, unsigned int reserve);
 void send_ccr_stop_with_param(struct ro_session *ro_session, unsigned int code, str* reason);
