@@ -46,7 +46,7 @@
 #include "third_party_reg.h"
 #include "../../core/msg_translator.h"
 
-usrloc_api_t isc_ulb;/*!< Structure containing pointers to usrloc functions*/
+extern usrloc_api_t isc_ulb;/*!< Structure containing pointers to usrloc functions*/
 
 /*! \brief
  * Combines all Path HF bodies into one string.
