@@ -116,7 +116,8 @@ typedef enum {
     IMS_REGISTRAR_CONTACT_EXPIRED, /**< A contact has expired and will be removed		*/
     IMS_REGISTRAR_CONTACT_UNREGISTERED, /**< User unregistered with Expires 0				*/
     IMS_REGISTRAR_CONTACT_UNREGISTERED_IMPLICIT, /**< User unregistered implicitly, ie not via explicit deregister	*/
-	IMS_REGISTRAR_SUBSEQUENT_SUBSCRIBE
+    IMS_REGISTRAR_SUBSEQUENT_SUBSCRIBE,
+    IMS_REGISTRAR_CONTACT_DEREGISTERED, /**< User's registration was terminated with RTR				*/
 } IMS_Registrar_events_enum_t;
 
 extern IMS_Registrar_events_enum_t IMS_Registrar_events;
