@@ -33,5 +33,7 @@ int janssonmod_array_size(struct sip_msg* msg, char* json_in,
 		char* path_in, char* dst);
 int janssonmod_get_helper(sip_msg_t* msg, str *path_s, str *src_s,
 		pv_spec_t *dst_pv);
+int jansson_xdecode(struct sip_msg* msg, char* src_in, char* xavp_in);
+int jansson_xencode(struct sip_msg* msg, char* xavp, char* dst);
 
 #endif
