@@ -26,6 +26,6 @@
 
 int jsonrpc_request(struct sip_msg* msg, char* method, char* params, char* cb_route, char* err_route, char* cb_pv);
 int jsonrpc_notification(struct sip_msg* msg, char* method, char* params);
-int cmd_pipe;
+extern int cmd_pipe;
 
 #endif /* _JSONRPC_REQUEST_H_ */
