@@ -122,6 +122,7 @@ typedef struct {
 	char* queue;
 	char* payload;
 	char* return_payload;
+	char* headers;
 	str* message_id;
 	int   return_code;
 	int   consumer;
@@ -273,10 +274,11 @@ void kz_amqp_destroy();
 int kz_amqp_add_connection(modparam_t type, void* val);
 
 int kz_amqp_publish(struct sip_msg* msg, char* exchange, char* routing_key, char* payload);
-int kz_amqp_publish_ex(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* _pub_flags);
+int kz_amqp_publish_ex(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* headers);
 int ki_kz_amqp_publish(sip_msg_t* msg, str* exchange, str* routing_key, str* payload);
-int kz_amqp_query(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* dst);
-int kz_amqp_query_ex(struct sip_msg* msg, char* exchange, char* routing_key, char* payload);
+int ki_kz_amqp_publish_hdrs(sip_msg_t* msg, str* exchange, str* routing_key, str* payload, str* headers);
+int kz_amqp_query(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* dst, char* headers);
+int kz_amqp_query_ex(struct sip_msg* msg, char* exchange, char* routing_key, char* payload, char* headers);
 int kz_amqp_subscribe(struct sip_msg* msg, char* payload);
 int ki_kz_amqp_subscribe(sip_msg_t* msg, str* payload);
 int kz_amqp_subscribe_simple(struct sip_msg* msg, char* exchange, char* exchange_type, char* queue_name, char* routing_key);
@@ -327,6 +329,8 @@ kz_amqp_queue_ptr kz_amqp_queue_new(str *name);
 kz_amqp_exchange_ptr kz_amqp_exchange_new(str *name, str* type);
 kz_amqp_routings_ptr kz_amqp_routing_new(char* routing);
 
+int add_amqp_headers (char * headers, amqp_basic_properties_t * props );
+
 static inline int kz_amqp_error(char const *context, amqp_rpc_reply_t x)
 {
 	amqp_connection_close_t *mconn;
