@@ -39,8 +39,10 @@
 
 #include "redis_client.h"
 
-#define redisCommandNR(a...) (int)({ void *__tmp; __tmp = redisCommand(a); \
-		if (__tmp) freeReplyObject(__tmp); __tmp ? 0 : -1;})
+#define redisCommandNR(a...) (int)({ \
+		void *__tmp; __tmp = redisCommand(a); \
+		if (__tmp) { freeReplyObject(__tmp); }; \
+		__tmp ? 0 : -1;})
 
 static redisc_server_t * _redisc_srv_list=NULL;
 
@@ -1163,6 +1165,7 @@ again:
 	if(res)
 	{
 		if (check_cluster_reply(res, &rsrv)) {
+			freeReplyObject(res);
 			goto again;
 		}
 		return res;
@@ -1173,6 +1176,7 @@ again:
 		res = redisCommandArgv(rsrv->ctxRedis, argc, argv, argvlen);
 		if (res) {
 			if (check_cluster_reply(res, &rsrv)) {
+				freeReplyObject(res);
 				goto again;
 			}
 		}
