@@ -35,7 +35,8 @@ struct cfg_group_p_usrloc	default_p_usrloc_cfg = {
 		DEFAULT_FAILOVER_LEVEL,	/* failover_level */
 		0,	/* db_ops_ruid */
 		1,	/* db_update_as_insert */
-		CONTACT_ONLY	/* matching_mode */
+		CONTACT_ONLY,	/* matching_mode */
+		0, /* utc_timestamps */
 	    };
 
 void	*p_usrloc_cfg = &default_p_usrloc_cfg;
@@ -53,5 +54,7 @@ cfg_def_t	p_usrloc_cfg_def[] = {
 		"Set this parameter if you want to do INSERT DB operations instead of UPDATE DB operations. "},
 	{"matching_mode",	CFG_VAR_INT | CFG_ATOMIC, 	0, 0, 0, 0,
 		"Specified which contact maching algorithm to be used (0 - Contact only / 1 - Contact and Call-ID / 2 - Contact and Path)"},
+	{"utc_timestamps",	CFG_VAR_INT | CFG_ATOMIC, 	0, 0, 0, 0,
+		"Expires and last_modified timestamps in UTC time format"},
 	{0, 0, 0, 0, 0, 0}
 };
