@@ -59,7 +59,6 @@ int reload_trusted_table(void)
 	db_val_t* val;
 
 	struct trusted_list **new_hash_table;
-	struct trusted_list **old_hash_table;
 	int i;
 	int priority;
 
@@ -160,15 +159,14 @@ int reload_trusted_table(void)
 
 	perm_dbf.free_result(perm_db_handle, res);
 
-	old_hash_table = *perm_trust_table;
 	*perm_trust_table = new_hash_table;
-	empty_hash_table(old_hash_table);
 
 	LM_DBG("trusted table reloaded successfully.\n");
 
 	return 1;
 }
 
+void perm_ht_timer(unsigned int ticks, void *);
 
 /*
  * Initialize data structures
@@ -227,6 +225,9 @@ int init_trusted(void)
 			goto error;
 		}
 
+		if(register_timer(perm_ht_timer, NULL, perm_trusted_table_interval) < 0)
+			goto error;
+
 		perm_dbf.close(perm_db_handle);
 		perm_db_handle = 0;
 	}
@@ -283,6 +284,22 @@ int init_child_trusted(int rank)
 }
 
 
+void perm_ht_timer(unsigned int ticks, void *param) {
+	if(perm_rpc_reload_time == NULL)
+		return;
+
+	if(*perm_rpc_reload_time != 0
+			&& *perm_rpc_reload_time > time(NULL) - perm_trusted_table_interval)
+			return;
+
+	LM_DBG("cleaning old trusted table\n");
+	if (*perm_trust_table == perm_trust_table_1) {
+		empty_hash_table(perm_trust_table_2);
+	} else {
+		empty_hash_table(perm_trust_table_1);
+	}
+}
+
 /*
  * Close connections and release memory
  */
