@@ -102,11 +102,15 @@ static int pres_dmq_init_proc()
 		}
 	}
 
+	if(publ_cache_mode==PS_PCACHE_RECORD && pres_subs_dbmode==NO_DB) {
+		goto finish;
+	}
+
 	if(!pa_db) {
 		LM_DBG("Initializing presence DB connection for pid (%d)\n", my_pid());
 
 		if(pa_dbf.init == 0) {
-			LM_ERR("dmq_worker_init: database not bound\n");
+			LM_ERR("database not bound\n");
 			return -1;
 		}
 
@@ -123,6 +127,7 @@ static int pres_dmq_init_proc()
 		}
 	}
 
+finish:
 	*pres_dmq_proc_init = 1;
 
 	LM_DBG("process initialization complete\n");
@@ -298,6 +303,9 @@ int pres_dmq_handle_msg(
 		} else if(strcmp(it->string, "body") == 0) {
 			p_body.s = it->valuestring;
 			p_body.len = strlen(it->valuestring);
+			if(p_body.len==0) {
+				p_body.s = NULL;
+			}
 		} else {
 			LM_ERR("unrecognized field in json object\n");
 			goto invalid;
