@@ -521,7 +521,7 @@ error:
 
 int http_get_rules_doc(str user, str domain, str *rules_doc)
 {
-	str uri;
+	str uri = STR_NULL;
 	xcap_doc_sel_t doc_sel;
 	char *doc = NULL;
 	xcap_serv_t *xs;
@@ -559,6 +559,8 @@ int http_get_rules_doc(str user, str domain, str *rules_doc)
 	rules_doc->s = doc;
 	rules_doc->len = doc ? strlen(doc) : 0;
 
+	pkg_free(uri.s);
+
 	return 0;
 
 error:
