@@ -43,7 +43,7 @@ extern htable_t* HashT;
 extern int HASH_SIZE;
 extern int min_expires;
 extern int pua_ul_publish;
-extern int default_expires;
+extern int pua_default_expires;
 extern str outbound_proxy;
 extern int check_remote_contact;
 extern int dbmode;
