@@ -122,6 +122,11 @@ static inline int is_myself(sip_uri_t *_puri)
 {
 	int ret;
 
+	if(_puri->host.len==0) {
+		/* catch uri without host (e.g., tel uri) */
+		return 0;
+	}
+
 	ret = check_self(&_puri->host,
 			_puri->port_no?_puri->port_no:SIP_PORT, 0);/* match all protos*/
 	if (ret < 0) return 0;
@@ -990,9 +995,10 @@ done:
 /*!
  * \brief Do loose routing as per RFC3261
  * \param _m SIP message
+ * \param _mode - 0: try loose or strict routing; 1: try loose routing only
  * \return negative on failure or preloaded, 1 on success
  */
-int loose_route(struct sip_msg* _m)
+int loose_route_mode(sip_msg_t* _m, int _mode)
 {
 	int ret;
 
@@ -1012,7 +1018,7 @@ int loose_route(struct sip_msg* _m)
 	} else if (ret == 1) {
 		return after_loose(_m, 1);
 	} else {
-		if (is_myself(&_m->parsed_uri)) {
+		if ((_mode==0) && (is_myself(&_m->parsed_uri))) {
 			return after_strict(_m);
 		} else {
 			return after_loose(_m, 0);
@@ -1020,6 +1026,16 @@ int loose_route(struct sip_msg* _m)
 	}
 }
 
+/*!
+ * \brief Do loose routing as per RFC3261
+ * \param _m SIP message
+ * \return negative on failure or preloaded, 1 on success
+ */
+int loose_route(struct sip_msg* _m)
+{
+	return loose_route_mode(_m, 0);
+}
+
 /**
  *
  */
