@@ -4,24 +4,7 @@
 #include <sys/uio.h>
 #include <string.h>
 
-#if defined(SHM_MEM) || defined(PKG_MALLOC) || defined(pkg_malloc)
-/* kamailio */
-# include "../../core/mem/mem.h"
-# include "../../core/str.h"
-# ifndef BENCODE_MALLOC
-# define BENCODE_MALLOC pkg_malloc
-# define BENCODE_FREE pkg_free
-# endif
-# define INLINE static inline
-#else
-/* rtpengine */
-# include "compat.h"
-# include "str.h"
-# ifndef BENCODE_MALLOC
-# define BENCODE_MALLOC malloc
-# define BENCODE_FREE free
-# endif
-#endif
+#include "compat.h"
 
 struct bencode_buffer;
 enum bencode_type;
