@@ -2569,6 +2569,8 @@ int receive_logging_json_msg(char *buf, unsigned int len,
 	str corrtmp = STR_NULL;
 	_capture_mode_data_t *c = NULL;
 
+	ipstr_dst[0] = '\0';
+	ipstr_src[0] = '\0';
 	c = capture_def;
 	if(!c) {
 		LM_ERR("no connection mode available to store data\n");
@@ -2580,6 +2582,8 @@ int receive_logging_json_msg(char *buf, unsigned int len,
 		return -1;
 	}
 
+	ipstr_dst[0] = '\0';
+	ipstr_src[0] = '\0';
 	memset(&sco, 0, sizeof(struct _sipcapture_object));
 	gettimeofday(&tvb, &tz);
 
