@@ -195,6 +195,10 @@ static str trace_local_ip = {NULL, 0};
 static db1_con_t *db_con = NULL; /*!< database connection */
 static db_func_t db_funcs;		  /*!< Database functions */
 
+int pv_parse_siptrace_name(pv_spec_t *sp, str *in);
+int pv_get_siptrace(sip_msg_t *msg, pv_param_t *param,
+		pv_value_t *res);
+
 /*! \brief
  * Exported functions
  */
@@ -276,6 +280,14 @@ stat_export_t siptrace_stats[] = {
 /* clang-format on */
 #endif
 
+static pv_export_t mod_pvs[] = {
+
+	{ {"siptrace", (sizeof("siptrace")-1)}, PVT_OTHER, pv_get_siptrace, 0,
+		pv_parse_siptrace_name, 0, 0, 0 },
+
+	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
+};
+
 /*! \brief module exports */
 /* clang-format off */
 struct module_exports exports = {
@@ -283,9 +295,9 @@ struct module_exports exports = {
 	DEFAULT_DLFLAGS, /*!< dlopen flags */
 	cmds,			/*!< exported functions */
 	params,			/*!< exported parameters */
-	0,				/*!< exported rpc functions */
-	0,				/*!< exported pseudo-variables */
-	0,				/*!< response function */
+	0,			/*!< exported rpc functions */
+	mod_pvs,		/*!< exported pseudo-variables */
+	0,			/*!< response function */
 	mod_init,		/*!< module initialization function */
 	child_init,		/*!< child initialization function */
 	destroy			/*!< destroy function */
@@ -2039,6 +2051,8 @@ static void trace_free_info(void* trace_info)
 	shm_free(trace_info);
 }
 
+static siptrace_data_t* siptrace_event_data = NULL;
+
 static int siptrace_exec_evcb_msg(siptrace_data_t *sto)
 {
 	int backup_rt;
@@ -2056,6 +2070,11 @@ static int siptrace_exec_evcb_msg(siptrace_data_t *sto)
 		return -1;
 	}
 
+	if(_siptrace_mode & SIPTRACE_MODE_URI) {
+		if(sip_trace_xheaders_write(sto) != 0)
+			return -1;
+	}
+
 	memset(&msg, 0, sizeof(sip_msg_t));
 	msg.buf = sto->body.s;
 	msg.len = sto->body.len;
@@ -2068,6 +2087,7 @@ static int siptrace_exec_evcb_msg(siptrace_data_t *sto)
 	set_route_type(EVENT_ROUTE);
 	init_run_actions_ctx(&ctx);
 
+	siptrace_event_data = sto;
 	if(_siptrace_evrt_msg_idx>=0) {
 		run_top_route(event_rt.rlist[_siptrace_evrt_msg_idx], &msg, &ctx);
 	} else {
@@ -2080,6 +2100,7 @@ static int siptrace_exec_evcb_msg(siptrace_data_t *sto)
 			sr_kemi_act_ctx_set(bctx);
 		}
 	}
+	siptrace_event_data = NULL;
 
 	free_sip_msg(&msg);
 	set_route_type(backup_rt);
@@ -2097,6 +2118,8 @@ int siptrace_net_data_recv(sr_event_param_t *evp)
 	sr_net_info_t *nd;
 	siptrace_data_t sto;
 	sip_msg_t tmsg;
+	int evcb_ret;
+	int ret = 0;
 
 	if(evp->data == 0)
 		return -1;
@@ -2139,10 +2162,15 @@ int siptrace_net_data_recv(sr_event_param_t *evp)
 
 	sto.dir = "in";
 
-	if(siptrace_exec_evcb_msg(&sto) == DROP_R_F) {
+	evcb_ret=siptrace_exec_evcb_msg(&sto);
+	if(evcb_ret < 0) {
+		ret = -1;
+		goto finish;
+	}
+	if(evcb_ret == DROP_R_F) {
 		/* drop() used in event_route - all done */
 		LM_DBG("skipping processing message due to drop\n");
-		return 0;
+		goto finish;
 	}
 
 	LM_DBG("processing message mode %d\n", _siptrace_mode);
@@ -2202,7 +2230,11 @@ afterdb:
 		trace_send_duplicate(sto.body.s, sto.body.len, NULL);
 	}
 
-	return 0;
+finish:
+	if(sip_trace_xheaders_free(&sto) != 0)
+		return -1;
+
+	return ret;
 }
 
 /**
@@ -2215,6 +2247,8 @@ int siptrace_net_data_sent(sr_event_param_t *evp)
 	siptrace_data_t sto;
 	sip_msg_t tmsg;
 	int proto;
+	int evcb_ret;
+	int ret = 0;
 
 	if(evp->data == 0)
 		return -1;
@@ -2247,7 +2281,7 @@ int siptrace_net_data_sent(sr_event_param_t *evp)
 		if(new_dst.send_sock->sock_str.len>=SIPTRACE_ADDR_MAX-1) {
 			LM_ERR("socket string is too large: %d\n",
 					new_dst.send_sock->sock_str.len);
-			goto error;
+			return -1;
 		}
 		strncpy(sto.fromip_buff, new_dst.send_sock->sock_str.s,
 				new_dst.send_sock->sock_str.len);
@@ -2270,10 +2304,15 @@ int siptrace_net_data_sent(sr_event_param_t *evp)
 
 	sto.dir = "out";
 
-	if(siptrace_exec_evcb_msg(&sto) == DROP_R_F) {
+	evcb_ret=siptrace_exec_evcb_msg(&sto);
+	if(evcb_ret < 0) {
+		ret = -1;
+		goto finish;
+	}
+	if(evcb_ret == DROP_R_F) {
 		/* drop() used in event_route - all done */
 		LM_DBG("skipping processing message due to drop\n");
-		return 0;
+		goto finish;
 	}
 
 	LM_DBG("processing message mode %d\n", _siptrace_mode);
@@ -2332,11 +2371,11 @@ afterdb:
 	if(_siptrace_mode & SIPTRACE_MODE_URI) {
 		trace_send_duplicate(sto.body.s, sto.body.len, NULL);
 	}
+finish:
+	if(sip_trace_xheaders_free(&sto) != 0)
+		return -1;
 
-	return 0;
-
-error:
-	return -1;
+	return ret;
 }
 
 /**
@@ -2482,6 +2521,148 @@ static sr_kemi_t sr_kemi_siptrace_exports[] = {
 
 	{ {0, 0}, {0, 0}, 0, NULL, { 0, 0, 0, 0, 0, 0 } }
 };
+
+/**
+ *
+ */
+int pv_get_siptrace(sip_msg_t *msg, pv_param_t *param,
+		pv_value_t *res)
+{
+	str host;
+	int port;
+	int proto;
+	str sproto;
+	if (siptrace_event_data==NULL) {
+		return pv_get_null(msg, param, res);
+	}
+
+	switch(param->pvn.u.isname.name.n) {
+		case 2: /* src_host */
+		case 4: /* src_port */
+		case 6: /* src_proto */
+		case 8: /* src_hostip */
+			if (parse_phostport(siptrace_event_data->fromip.s, &host.s, &host.len, &port, &proto)!=0) {
+				LM_ERR("invalid src_addr: %.*s\n", siptrace_event_data->fromip.len, siptrace_event_data->fromip.s);
+				return pv_get_null(msg, param, res);
+			}
+		break;
+		case 3: /* dst_host */
+		case 5: /* dst_port */
+		case 7: /* dst_proto */
+		case 9: /* dst_hostip */
+			if (parse_phostport(siptrace_event_data->toip.s, &host.s, &host.len, &port, &proto)!=0) {
+				LM_ERR("invalid dst_addr: %.*s\n", siptrace_event_data->toip.len, siptrace_event_data->toip.s);
+				return pv_get_null(msg, param, res);
+			}
+		break;
+	}
+
+	switch(param->pvn.u.isname.name.n) {
+		case 6: /* src_proto */
+		case 7: /* dst_proto */
+			if(get_valid_proto_string(proto, 0, 0, &sproto)<0)
+			{
+				sproto.s = "none";
+				sproto.len = 4;
+			}
+		break;
+	}
+
+	switch(param->pvn.u.isname.name.n) {
+		case 8: /* src_hostip */
+		case 9: /* dst_hostip */
+
+			/* now IPv6 address has brakets that not wanted in IP address operations */
+			if(host.s[0] == '[' && host.s[host.len-1] == ']') {
+				host.s++;
+				host.len = host.len - 2;
+			}
+		break;
+	}
+
+	switch(param->pvn.u.isname.name.n) {
+		case 0: /* src_addr */
+			return pv_get_strval(msg, param, res, &siptrace_event_data->fromip);
+		case 1: /* dst_addr */
+			return pv_get_strval(msg, param, res, &siptrace_event_data->toip);
+		case 2: /* src_host */
+			return pv_get_strval(msg, param, res, &host);
+		case 3: /* dst_host */
+			return pv_get_strval(msg, param, res, &host);
+		case 4: /* src_port */
+			return pv_get_sintval(msg, param, res, port);
+		case 5: /* dst_port */
+			return pv_get_sintval(msg, param, res, port);
+		case 6: /* src_proto */
+			return pv_get_strintval(msg, param, res, &sproto, proto);
+		case 7: /* dst_proto */
+			return pv_get_strintval(msg, param, res, &sproto, proto);
+		case 8: /* src_hostip */
+			return pv_get_strval(msg, param, res, &host);
+		case 9: /* dst_hostip */
+			return pv_get_strval(msg, param, res, &host);
+		case 10: /* direction */
+			return pv_get_strzval(msg, param, res, siptrace_event_data->dir);
+		default:
+			LM_ERR("unexpected config param\n");
+			return pv_get_null(msg, param, res);
+	}
+}
+
+/**
+ *
+ */
+int pv_parse_siptrace_name(pv_spec_t *sp, str *in)
+{
+	if(sp==NULL || in==NULL || in->len<=0)
+		return -1;
+
+	switch(in->len)
+	{
+		case 8:
+			if(strncmp(in->s, "src_addr", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 0;
+			else if(strncmp(in->s, "dst_addr", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 1;
+			else if(strncmp(in->s, "src_host", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 2;
+			else if(strncmp(in->s, "dst_host", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 3;
+			else if(strncmp(in->s, "src_port", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 4;
+			else if(strncmp(in->s, "dst_port", 8)==0)
+				sp->pvp.pvn.u.isname.name.n = 5;
+			else goto error;
+		break;
+		case 9:
+			if(strncmp(in->s, "src_proto", 9)==0)
+				sp->pvp.pvn.u.isname.name.n = 6;
+			else if(strncmp(in->s, "dst_proto", 9)==0)
+				sp->pvp.pvn.u.isname.name.n = 7;
+			else goto error;
+		break;
+		case 10:
+			if(strncmp(in->s, "src_hostip", 10)==0)
+				sp->pvp.pvn.u.isname.name.n = 8;
+			else if(strncmp(in->s, "dst_hostip", 10)==0)
+				sp->pvp.pvn.u.isname.name.n = 9;
+			else if(strncmp(in->s, "direction", 10)==0)
+				sp->pvp.pvn.u.isname.name.n = 10;
+			else goto error;
+		break;
+		default:
+			goto error;
+	}
+	sp->pvp.pvn.type = PV_NAME_INTSTR;
+	sp->pvp.pvn.u.isname.type = 0;
+
+	return 0;
+
+error:
+	LM_ERR("unknown PV snd name %.*s\n", in->len, in->s);
+	return -1;
+}
+
 /* clang-format on */
 
 /**
