@@ -35,7 +35,7 @@
 
 #include "sql_api.h"
 
-#define SQLOPS_RESULT_MAXSIZE 32
+extern int sqlops_results_maxsize;
 
 sql_con_t *_sql_con_root = NULL;
 sql_result_t *_sql_result_root = NULL;
@@ -225,7 +225,7 @@ sql_result_t* sql_get_result(str *name)
 		i++;
 		sr = sr->next;
 	}
-	if(i>SQLOPS_RESULT_MAXSIZE)
+	if(i>sqlops_results_maxsize)
 	{
 		LM_ERR("too many result containers defined\n");
 		return NULL;
@@ -954,7 +954,7 @@ int sqlops_do_xquery(sip_msg_t *msg, str *scon, str *squery, str *xavp)
 	if(sql_exec_xquery(msg, con, squery, xavp)<0)
 		goto error;
 
-	return 0;
+	return 1;
 error:
 	return -1;
 }
