@@ -589,10 +589,20 @@ int tls_pre_init(void)
 	void (*ff)(void *, const char *, int) = NULL;
 #endif
 
+#ifdef KSR_LIBSSL_STATIC
+	LM_INFO("libssl linked mode: static\n");
+#endif
+
 	/*
 	 * this has to be called before any function calling CRYPTO_malloc,
 	 * CRYPTO_malloc will set allow_customize in openssl to 0
 	 */
+	CRYPTO_get_mem_functions(&mf, &rf, &ff);
+	LM_DBG("initial memory functions - malloc: %p realloc: %p free: %p\n",
+			mf, rf, ff);
+	mf = NULL;
+	rf = NULL;
+	ff = NULL;
 #ifdef TLS_MALLOC_DBG
 	if (!CRYPTO_set_mem_ex_functions(ser_malloc, ser_realloc, ser_free)) {
 #else
@@ -602,10 +612,14 @@ int tls_pre_init(void)
 		CRYPTO_get_mem_functions(&mf, &rf, &ff);
 		LM_ERR("libssl current mem functions - m: %p r: %p f: %p\n",
 					mf, rf, ff);
+		LM_ERR("module mem functions - m: %p r: %p f: %p\n",
+					ser_malloc, ser_realloc, ser_free);
 		LM_ERR("Be sure tls module is loaded before any other module using"
 				" libssl (can be loaded first to be safe)\n");
 		return -1;
 	}
+	LM_DBG("updated memory functions - malloc: %p realloc: %p free: %p\n",
+			ser_malloc, ser_realloc, ser_free);
 #endif /* LIBRESSL_VERSION_NUMBER */
 
 	if (tls_init_locks()<0)
@@ -646,6 +660,7 @@ int tls_h_mod_init_f(void)
 {
 	/*struct socket_info* si;*/
 	long ssl_version;
+	const char *ssl_version_txt;
 #if OPENSSL_VERSION_NUMBER < 0x010100000L && !defined(LIBRESSL_VERSION_NUMBER)
 	int lib_kerberos;
 	int lib_zlib;
@@ -668,7 +683,15 @@ int tls_h_mod_init_f(void)
 #if OPENSSL_VERSION_NUMBER < 0x00907000L
 	LM_WARN("You are using an old version of OpenSSL (< 0.9.7). Upgrade!\n");
 #endif
+
+#if OPENSSL_VERSION_NUMBER < 0x010100000L
 	ssl_version=SSLeay();
+	ssl_version_txt=SSLeay_version(SSLEAY_VERSION);
+#else
+	ssl_version=OpenSSL_version_num();
+	ssl_version_txt=OpenSSL_version(OPENSSL_VERSION);
+#endif
+
 	/* check if version have the same major minor and fix level
 	 * (e.g. 0.9.8a & 0.9.8c are ok, but 0.9.8 and 0.9.9x are not)
 	 * - values is represented as 0xMMNNFFPPS: major minor fix patch status
@@ -680,7 +703,7 @@ int tls_h_mod_init_f(void)
 				" compiled \"%s\" (0x%08lx).\n"
 				" Please make sure a compatible version is used"
 				" (tls_force_run in kamailio.cfg will override this check)\n",
-				SSLeay_version(SSLEAY_VERSION), ssl_version,
+				ssl_version_txt, ssl_version,
 				OPENSSL_VERSION_TEXT, (long)OPENSSL_VERSION_NUMBER);
 		if (cfg_get(tls, tls_cfg, force_run))
 			LM_WARN("tls_force_run turned on, ignoring "
