@@ -285,7 +285,7 @@ static int tls_complete_init(struct tcp_connection* c)
 	data->state = state;
 
 	if (unlikely(data->ssl == 0 || data->rwbio == 0)) {
-		TLS_ERR("Failed to create SSL or BIO structure:");
+		TLS_ERR_SSL("Failed to create SSL or BIO structure:", data->ssl);
 		if (data->ssl)
 			SSL_free(data->ssl);
 		if (data->rwbio)
@@ -446,7 +446,7 @@ EVP_PKEY * tls_lookup_private_key(SSL_CTX*);
 int tls_accept(struct tcp_connection *c, int* error)
 {
 	int ret;
-	SSL *ssl;
+	SSL *ssl = NULL;
 	X509* cert;
 	struct tls_extra_data* tls_c;
 	int tls_log;
@@ -737,7 +737,7 @@ void tls_h_tcpconn_close_f(struct tcp_connection *c, int fd)
 			tls_mbuf_init(&rd, 0, 0); /* no read */
 			tls_mbuf_init(&wr, wr_buf, sizeof(wr_buf));
 			if (tls_set_mbufs(c, &rd, &wr)==0) {
-				tls_shutdown(c); /* shudown only on succesfull set fd */
+				tls_shutdown(c); /* shutdown only on successful set fd */
 				/* write as much as possible and update wr.
 				 * Since this is a close, we don't want to queue the write
 				 * (if it can't write immediately, just fail silently)
@@ -792,7 +792,7 @@ int tls_h_encode_f(struct tcp_connection *c,
 						snd_flags_t* send_flags)
 {
 	int n, offs;
-	SSL* ssl;
+	SSL* ssl = NULL;
 	struct tls_extra_data* tls_c;
 	static unsigned char wr_buf[TLS_WR_MBUF_SZ];
 	struct tls_mbuf rd, wr;
@@ -929,7 +929,7 @@ redo_wr:
 			case SSL_ERROR_SSL:
 				/* protocol level error */
 				ERR("protocol level error\n");
-				TLS_ERR(err_src);
+				TLS_ERR_SSL(err_src, ssl);
 				memset(ip_buf, 0, sizeof(buf));
 				ip_addr2sbuf(&(c->rcv.src_ip), ip_buf, sizeof(ip_buf));
 				ERR("source IP: %s\n", ip_buf);
@@ -970,7 +970,7 @@ redo_wr:
 				}
 				goto error;
 			default:
-				TLS_ERR(err_src);
+				TLS_ERR_SSL(err_src, ssl);
 				BUG("unexpected SSL error %d\n", ssl_error);
 				goto bug;
 		}
@@ -1053,6 +1053,7 @@ int tls_h_read_f(struct tcp_connection* c, rd_conn_flags_t* flags)
 	int x;
 	int tls_dbg;
 
+	ssl = NULL;
 	TLS_RD_TRACE("(%p, %p (%d)) start (%s -> %s:%d*)\n",
 					c, flags, *flags,
 					su2a(&c->rcv.src_su, sizeof(c->rcv.src_su)),
@@ -1327,7 +1328,7 @@ ssl_read_skipped:
 		case SSL_ERROR_SSL:
 			/* protocol level error */
 			ERR("protocol level error\n");
-			TLS_ERR(err_src);
+			TLS_ERR_SSL(err_src, ssl);
 			memset(ip_buf, 0, sizeof(ip_buf));
 			ip_addr2sbuf(&(c->rcv.src_ip), ip_buf, sizeof(ip_buf));
 			ERR("src addr: %s:%d\n", ip_buf, c->rcv.src_port);
@@ -1368,7 +1369,7 @@ ssl_read_skipped:
 			}
 			goto error;
 		default:
-			TLS_ERR(err_src);
+			TLS_ERR_SSL(err_src, ssl);
 			BUG("unexpected SSL error %d\n", ssl_error);
 			goto bug;
 	}
