@@ -44,7 +44,6 @@
 
 #include "../../core/clist.h"
 #include "../../core/parser/msg_parser.h"
-#include "../../core/md5utils.h"
 #include "../../core/usr_avp.h"
 #include "../../core/xavp.h"
 #include "../../core/timer.h"
@@ -309,6 +308,8 @@ typedef struct tm_xdata
 	struct usr_avp *domain_avps_from;
 	struct usr_avp *domain_avps_to;
 	sr_xavp_t *xavps_list;
+	sr_xavp_t *xavus_list;
+	sr_xavp_t *xavis_list;
 } tm_xdata_t;
 
 
@@ -325,6 +326,8 @@ typedef struct tm_xlinks
 	struct usr_avp **domain_avps_from;
 	struct usr_avp **domain_avps_to;
 	sr_xavp_t **xavps_list;
+	sr_xavp_t **xavus_list;
+	sr_xavp_t **xavis_list;
 } tm_xlinks_t;
 
 
@@ -398,6 +401,8 @@ typedef struct cell
 	struct usr_avp *domain_avps_from;
 	struct usr_avp *domain_avps_to;
 	sr_xavp_t *xavps_list;
+	sr_xavp_t *xavus_list;
+	sr_xavp_t *xavis_list;
 
 	/* protection against concurrent reply processing */
 	ser_lock_t reply_mutex;
