@@ -21,8 +21,12 @@
 #include "../../core/rpc.h"
 
 
+int tm_rpc_response_list_init(void);
+int tm_rpc_response_list_destroy(void);
+
 void rpc_t_uac_start(rpc_t* rpc, void* c);
 void rpc_t_uac_wait(rpc_t* rpc, void* c);
+void rpc_t_uac_wait_block(rpc_t* rpc, void* c);
 
 int t_uac_send(str *method, str *ruri, str *nexthop, str *send_socket,
 		str *headers, str *body);
