@@ -13,8 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
@@ -28,24 +28,6 @@
 #include "h_table.h"
 #include "t_reply.h"
 
-
-#define CSEQ "CSeq: "
-#define CSEQ_LEN (sizeof(CSEQ)-1)
-#define TO "To: "
-#define TO_LEN (sizeof(TO)-1)
-#define CALLID "Call-ID: "
-#define CALLID_LEN (sizeof(CALLID)-1)
-#define FROM "From: "
-#define FROM_LEN (sizeof(FROM)-1)
-#define FROMTAG ";tag="
-#define FROMTAG_LEN (sizeof(FROMTAG)-1)
-#define TOTAG ";tag="
-#define TOTAG_LEN (sizeof(TOTAG)-1)
-#define MAXFWD_VALUE "70"
-#define MAXFWD_HEADER "Max-Forwards: " MAXFWD_VALUE CRLF
-#define MAXFWD_HEADER_LEN (sizeof(MAXFWD_HEADER) - 1)
-
-
 char *build_local(struct cell *Trans, unsigned int branch,
 	unsigned int *len, char *method, int method_len, str *to
 	, struct cancel_reason* reason
@@ -57,7 +39,7 @@ char *build_local_reparse(struct cell *Trans, unsigned int branch,
 	);
 
 char *build_uac_request(  str msg_type, str dst, str from,
-	str fromtag, int cseq, str callid, str headers, 
+	str fromtag, int cseq, str callid, str headers,
 	str body, int branch,
 	struct cell *t, unsigned int *len);
 
@@ -73,7 +55,7 @@ char *build_uac_cancel(str *headers,str *body,struct cell *cancelledT,
  * request should be send. The function is used by tm when it generates
  * local ACK to 200 OK (on behalf of applications using uac
  */
-char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans, 
+char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans,
 					unsigned int branch, str *hdrs, str *body,
 					unsigned int *len, struct dest_info* dst);
 
@@ -81,8 +63,8 @@ char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans,
 /*
  * Create a request
  */
-char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int branch, 
-		    struct cell *t, int* len, struct dest_info* dst);
+char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int branch,
+		struct cell *t, int* len, struct dest_info* dst);
 
 
 int t_calc_branch(struct cell *t,
@@ -92,4 +74,7 @@ int t_calc_branch(struct cell *t,
 char* print_callid_mini(char* target, str callid);
 char* print_cseq_mini(char* target, str* cseq, str* method);
 
+typedef void (*t_uas_request_clean_parsed_f)(tm_cell_t *t);
+void t_uas_request_clean_parsed(tm_cell_t *t);
+
 #endif
