@@ -432,6 +432,7 @@ static param_export_t params[]={
 	{"ruri_matching",       PARAM_INT, &default_tm_cfg.ruri_matching         },
 	{"via1_matching",       PARAM_INT, &default_tm_cfg.via1_matching         },
 	{"callid_matching",     PARAM_INT, &default_tm_cfg.callid_matching       },
+	{"callid_cseq_matching",PARAM_INT, &default_tm_cfg.callid_cseq_matching  },
 	{"fr_timer",            PARAM_INT, &default_tm_cfg.fr_timeout            },
 	{"fr_inv_timer",        PARAM_INT, &default_tm_cfg.fr_inv_timeout        },
 	{"wt_timer",            PARAM_INT, &default_tm_cfg.wait_timeout          },
@@ -485,6 +486,7 @@ static param_export_t params[]={
 	{"event_callback_lres_sent", PARAM_STR, &_tm_event_callback_lres_sent    },
 	{"exec_time_check" ,    PARAM_INT, &tm_exec_time_check_param             },
 	{"reply_relay_mode",    PARAM_INT, &tm_reply_relay_mode                  },
+	{"enable_uac_fr",       PARAM_INT, &default_tm_cfg.enable_uac_fr         },
 	{0,0,0}
 };
 
@@ -2928,7 +2930,7 @@ static int ki_t_relay_to_proto(sip_msg_t *msg, str *sproto)
 			proto = PROTO_UDP;
 		} else if (strncasecmp(sproto->s, "TCP", 3) == 0) {
 			proto = PROTO_TCP;
-		} else if (strncasecmp(sproto->s, "TLS", 3)) {
+		} else if (strncasecmp(sproto->s, "TLS", 3) == 0) {
 			proto = PROTO_TLS;
 		} else {
 			LM_ERR("bad protocol specified <%s>\n", sproto->s);
@@ -2953,7 +2955,7 @@ static int ki_t_relay_to_proto_addr(sip_msg_t *msg, str *sproto, str *host, int
 			proto = PROTO_UDP;
 		} else if (strncasecmp(sproto->s, "TCP", 3) == 0) {
 			proto = PROTO_TCP;
-		} else if (strncasecmp(sproto->s, "TLS", 3)) {
+		} else if (strncasecmp(sproto->s, "TLS", 3) == 0) {
 			proto = PROTO_TLS;
 		} else {
 			LM_ERR("bad protocol specified <%s>\n", sproto->s);
