@@ -46,6 +46,8 @@
 #include "tps_msg.h"
 #include "tps_storage.h"
 
+#define TPS_METHODS_NOCONTACT (METHOD_CANCEL|METHOD_BYE|METHOD_PRACK)
+
 extern int _tps_param_mask_callid;
 extern int _tps_contact_mode;
 extern str _tps_cparam_name;
@@ -582,6 +584,10 @@ int tps_reinsert_contact(sip_msg_t *msg, tps_data_t *ptsd, str *hbody)
 {
 	str hname = str_init("Contact");
 
+	if (get_cseq(msg)->method_id & TPS_METHODS_NOCONTACT) {
+		return 0;
+	}
+
 	if(tps_add_headers(msg, &hname, hbody, 0)<0) {
 		return -1;
 	}
@@ -803,6 +809,7 @@ int tps_request_received(sip_msg_t *msg, int dialog)
 	uint32_t direction = TPS_DIR_DOWNSTREAM;
 	int ret;
 	int use_branch = 0;
+	unsigned int metid = 0;
 
 	LM_DBG("handling incoming request\n");
 
@@ -835,10 +842,11 @@ int tps_request_received(sip_msg_t *msg, int dialog)
 	if(tps_storage_load_dialog(msg, &mtsd, &stsd) < 0) {
 		goto error;
 	}
-	if(((get_cseq(msg)->method_id) & (METHOD_BYE|METHOD_PRACK|METHOD_UPDATE))
+	metid = get_cseq(msg)->method_id;
+	if((metid & (METHOD_BYE|METHOD_INFO|METHOD_PRACK|METHOD_UPDATE))
 			&& stsd.b_contact.len <= 0) {
 		/* no B-side contact, look for INVITE transaction record */
-		if((get_cseq(msg)->method_id) & (METHOD_UPDATE)) {
+		if(metid & (METHOD_BYE|METHOD_UPDATE)) {
 			/* detect direction - via from-tag */
 			if(tps_dlg_detect_direction(msg, &stsd, &direction) < 0) {
 				goto error;
@@ -861,10 +869,9 @@ int tps_request_received(sip_msg_t *msg, int dialog)
 		mtsd.direction = direction;
 	}
 
-
 	tps_storage_lock_release(&lkey);
 
-	if(use_branch) {
+	if(use_branch && direction == TPS_DIR_DOWNSTREAM) {
 		nuri = stsd.b_contact;
 	} else {
 		if(direction == TPS_DIR_UPSTREAM) {
@@ -882,7 +889,7 @@ int tps_request_received(sip_msg_t *msg, int dialog)
 		}
 	}
 
-	if(use_branch) {
+	if(use_branch && direction == TPS_DIR_DOWNSTREAM) {
 		if(tps_reappend_route(msg, &stsd, &stsd.s_rr, 1) < 0) {
 			LM_ERR("failed to reappend s-route\n");
 			return -1;
@@ -916,7 +923,7 @@ int tps_request_received(sip_msg_t *msg, int dialog)
 				goto error;
 			}
 		}
-		if((get_cseq(msg)->method_id)&(METHOD_SUBSCRIBE)) {
+		if(metid & METHOD_SUBSCRIBE) {
 			if(tps_storage_update_dialog(msg, &mtsd, &stsd, TPS_DBU_CONTACT|TPS_DBU_TIME)<0) {
 				goto error;
 			}
@@ -1160,6 +1167,11 @@ int tps_response_sent(sip_msg_t *msg)
 				&& msg->contact==NULL) {
 		contact_keep = 1;
 	}
+	if(contact_keep==0 && msg->first_line.u.reply.statuscode>=400
+				&& msg->first_line.u.reply.statuscode<500
+				&& msg->contact==NULL) {
+		contact_keep = 1;
+	}
 	if(contact_keep==0) {
 		tps_remove_headers(msg, HDR_CONTACT_T);
 		if(direction==TPS_DIR_DOWNSTREAM) {
