@@ -464,7 +464,7 @@ ws_connection_t *wsconn_get(int id)
 	for(wsc = wsconn_id_hash[id_hash]; wsc; wsc = wsc->id_next) {
 		if(wsc->id == id) {
 			wsconn_ref(wsc);
-			LM_DBG("wsconn_get returns wsc [%p] refcnt [%d]\n", wsc,
+			LM_DBG("wsconn_get id [%d] returns wsc [%p] refcnt [%d]\n", id, wsc,
 					atomic_get(&wsc->refcnt));
 
 			WSCONN_UNLOCK;
@@ -679,6 +679,7 @@ void ws_timer(unsigned int ticks, void *param)
 	ws_connection_list_t rmlist;
 	ws_connection_t *wsc;
 	ws_connection_t *next;
+	struct tcp_connection *con = NULL;
 	ticks_t nticks;
 	int h;
 
@@ -695,6 +696,15 @@ void ws_timer(unsigned int ticks, void *param)
 				wsconn_detach_connection(wsc);
 				wsc->id_next = rmlist.head;
 				rmlist.head = wsc;
+			} else if(wsc->state != WS_S_REMOVING) {
+				con = tcpconn_get(wsc->id, 0, 0, 0, 0);
+				if(con == NULL) {
+					LM_DBG("ws structure without active tcp connection\n");
+					wsc->state = WS_S_REMOVING;
+					wsc->rmticks = get_ticks();
+				} else {
+					tcpconn_put(con);
+				}
 			}
 			wsc = next;
 		}
@@ -756,8 +766,10 @@ static int ws_rpc_add_node(
 
 		tcpconn_put(con);
 		return 1;
-	} else
+	} else {
+		LM_DBG("ws structure [%p] without an active tcp connection\n", wsc);
 		return 0;
+	}
 }
 
 void ws_rpc_dump(rpc_t *rpc, void *ctx)
